@@ -3,13 +3,14 @@
 #include <obs-frontend-api.h>
 #include <QDockWidget>
 #include <QTextEdit>
+#include <QTimer>
 #include "obs.hpp"
 
 class SceneNotesDock : public QDockWidget {
 	Q_OBJECT
 
 private:
-	bool show_preview;
+	bool show_preview = false;
 	QTextEdit *textEdit;
 private slots:
 	void InsertTime();
@@ -18,4 +19,6 @@ public:
 	~SceneNotesDock();
 	void LoadNotes();
 	obs_hotkey_id insertTime;
+	obs_hotkey_pair_id toggleAutoScroll;
+	QTimer timer;
 };
